@@ -1,30 +1,38 @@
+import sys
 import typing
 
+if sys.version_info < (3, 10):
+    from typing_extensions import ParamSpec
+else:
+    from typing import ParamSpec
+
+
 import click
+import httpx
 import typer
 
 from neosctl import schema, util
 from neosctl.util import (
     check_profile_exists,
     constant,
+    exit_with_output,
     get_user_profile,
     is_success_response,
     process_response,
     read_config_dotfile,
-    send_output,
     upsert_config,
 )
 
 app = typer.Typer()
 
 
-def auth_url(iam_api_url: str) -> str:
+def _auth_url(iam_api_url: str) -> str:
     return "{}".format(iam_api_url.rstrip("/"))
 
 
-def _check_refresh_token_exists(ctx: typer.Context):
+def _check_refresh_token_exists(ctx: click.Context) -> bool:
     if ctx.obj.profile.refresh_token == "":  # nosec: B105
-        send_output(
+        raise exit_with_output(
             msg=f"You need to login. Run neosctl -p {ctx.obj.profile_name} auth login",
             exit_code=1,
         )
@@ -32,12 +40,39 @@ def _check_refresh_token_exists(ctx: typer.Context):
     return True
 
 
-def ensure_login(method):
-    def check_access_token(*args, **kwargs):
+def _refresh_token(ctx: click.Context) -> httpx.Response:
+    check_profile_exists(ctx)
+    _check_refresh_token_exists(ctx)
+
+    r = util.post(
+        ctx,
+        f"{_auth_url(ctx.obj.get_iam_api_url())}/refresh",
+        json={"refresh_token": ctx.obj.profile.refresh_token},
+    )
+
+    if not is_success_response(r):
+        process_response(r)
+
+    upsert_config(ctx, _update_profile(ctx, schema.Auth(**r.json())))
+
+    return r
+
+
+P = ParamSpec("P")
+
+
+def ensure_login(method: typing.Callable[P, httpx.Response]) -> typing.Callable[P, httpx.Response]:
+    """Capture authentication errors and retry requests.
+
+    On request failure check if the response is a 401, and attempt to refresh the access_token.
+    Retry the request with refreshed token, on subsequent failure, return.
+    """
+
+    def check_access_token(*args: P.args, **kwargs: P.kwargs) -> httpx.Response:
         ctx = args[0]
-        if not isinstance(ctx, click.core.Context):
+        if not isinstance(ctx, click.Context):
             # Developer reminder
-            msg = "First argument should be typer.Context instance"
+            msg = "First argument should be click.Context instance"
             raise TypeError(msg)
 
         r = method(*args, **kwargs)
@@ -49,7 +84,7 @@ def ensure_login(method):
         if r.status_code == constant.UNAUTHORISED_CODE:
             data = r.json()
             if "code" in data and data["code"].startswith("A0"):
-                refresh_token(ctx)
+                _refresh_token(ctx)
 
                 # Refresh the context
                 c = read_config_dotfile()
@@ -64,9 +99,9 @@ def ensure_login(method):
 
 
 def _update_profile(
-    ctx: typer.Context,
+    ctx: click.Context,
     auth: schema.Auth = schema.Auth(),
-):
+) -> schema.Profile:
     return schema.Profile(
         gateway_api_url=ctx.obj.gateway_api_url,
         registry_api_url=ctx.obj.registry_api_url,
@@ -82,8 +117,8 @@ def _update_profile(
 @app.command()
 def login(
     ctx: typer.Context,
-    password: typing.Optional[str] = typer.Option(None, "--password", "-p"),
-):
+    password: typing.Optional[str] = typer.Option(None, "--password", "-p", callback=util.sanitize),
+) -> None:
     """Login to neos."""
     check_profile_exists(ctx)
 
@@ -98,7 +133,7 @@ def login(
 
     r = util.post(
         ctx,
-        f"{auth_url(ctx.obj.get_iam_api_url())}/login",
+        f"{_auth_url(ctx.obj.get_iam_api_url())}/login",
         json={"user": ctx.obj.profile.user, "password": password},
     )
 
@@ -107,14 +142,14 @@ def login(
 
     upsert_config(ctx, _update_profile(ctx, schema.Auth(**r.json())))
 
-    send_output(
+    raise exit_with_output(
         msg="Login success",
         exit_code=0,
     )
 
 
 @app.command()
-def logout(ctx: typer.Context):
+def logout(ctx: typer.Context) -> None:
     """Logout from neos."""
     check_profile_exists(ctx)
 
@@ -122,7 +157,7 @@ def logout(ctx: typer.Context):
 
     r = util.post(
         ctx,
-        f"{auth_url(ctx.obj.get_iam_api_url())}/logout",
+        f"{_auth_url(ctx.obj.get_iam_api_url())}/logout",
         json={"refresh_token": ctx.obj.profile.refresh_token},
     )
 
@@ -131,25 +166,7 @@ def logout(ctx: typer.Context):
 
     upsert_config(ctx, _update_profile(ctx, schema.Auth()))
 
-    send_output(
+    raise exit_with_output(
         msg="Logout success",
         exit_code=0,
     )
-
-
-def refresh_token(ctx: typer.Context):
-    check_profile_exists(ctx)
-    _check_refresh_token_exists(ctx)
-
-    r = util.post(
-        ctx,
-        f"{auth_url(ctx.obj.get_iam_api_url())}/refresh",
-        json={"refresh_token": ctx.obj.profile.refresh_token},
-    )
-
-    if not is_success_response(r):
-        process_response(r)
-
-    upsert_config(ctx, _update_profile(ctx, schema.Auth(**r.json())))
-
-    return r
