@@ -1,10 +1,16 @@
 import json
 import mimetypes
 import pathlib
+import sys
 import typing
 
 import httpx
-import jq
+
+if sys.platform != "win32":
+    import jq
+else:
+    jq = None
+
 import typer
 
 from neosctl import constant, schema, util
