@@ -102,6 +102,8 @@ class CreateSource(pydantic.BaseModel):
 
 class ExternalDatabaseConnectionDetails(pydantic.BaseModel):
     type: Literal["external_database"]  # noqa: A003
+    engine: str
+    schema_: str = pydantic.Field(..., alias="schema")
     host: str
     port: int
     database: str
